package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.Resource;
import org.springframework.hateoas.Resources;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RestController;
import java.util.List;
import java.util.stream.Collectors;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

@RestController
public class TaskController {
    @Autowired
    private EmployeeService employeeService;

    @GetMapping("/employees/{employeeId}/tasks")
    public Resources<Resource<Task>> getEmployeeTasks(@PathVariable long employeeId) {
        List<Task> tasks = employeeService.getTasksByEmployeeId(employeeId);
        List<Resource<Task>> taskResources = tasks.stream()
                                                  .map(TaskController::createTaskResource)
                                                  .collect(Collectors.toList());
        return new Resources<>(taskResources);
    }

    @GetMapping("/tasks/{taskId}")
    public Resource<Task> getTask(@PathVariable long taskId) {
        Task task = employeeService.getTaskByTaskId(taskId);
        Link selfRel = linkTo(methodOn(TaskController.class).getTask(taskId)).withSelfRel();
        Resource<Task> taskResource = new Resource<>(task, selfRel);
        taskResource.add(createEmployeeLink(task.getEmployee().getEmployeeId()));
        return taskResource;
    }

    @GetMapping("/tasks")
    public Resources<Resource<Task>> getAllTasks() {
        List<Task> tasks = employeeService.getAllTasks();
        Link selfRel = linkTo(methodOn(TaskController.class).getAllTasks()).withSelfRel();
        List<Resource<Task>> taskResources = tasks.stream()
                                                  .map(TaskController::createTaskResource)
                                                  .collect(Collectors.toList());
        return new Resources<>(taskResources, selfRel);
    }

    private static Resource<Task> createTaskResource(Task t) {
        Link selfRel = linkTo(methodOn(TaskController.class).getTask(t.getTaskId())).withSelfRel();
        return new Resource<>(t, selfRel,
                createEmployeeLink(t.getEmployee().getEmployeeId()));
    }

    static Link createEmployeeLink(long employeeId) {
        return linkTo(methodOn(EmployeeController.class).getEmployeeById(employeeId))
                .withRel("employee");
    }
}